/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 1995, 1996, 1997, 1998 John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@johncon.com (John Conover)

------------------------------------------------------------------------------

The pdict program uses the soundex algorithm to search a text version
of a standard English dictionary for words that sound like a word
supplied on the command line. The format of the dictionary is one word
per line. Each line must be EOL terminated, ie., with a '\n'.

The word on the command line is converted to its soundex
representation, then sequentially compared to the soundex
representation of each word in the dictionary. If the two compare, the
dictionary word is printed to the standard output.

The soundex algorithm is a mechanical phonetic translation system for
the English language, and converts English words into a corresponding
phonetic code for the word. The algorithm is as follows:

    for each character in a word:

        if the character is the first character of a word

            1) do nothing

        else

            2) replace consecutive sequences of the labials, (ie., the
            characters, B, F, P, V,) with the character '1'

            3) replace consecutive sequences of the gutterals and
            sibilants, (ie., the characters, C, G, J, K, Q, S, X, Z,)
            with the character '2'

            4) replace consecutive sequences of the dentals, (ie., the
            characters, D, T,) with the character '3'

            5) replace consecutive sequences of the longliquids, (ie.,
            the character, L,) with the character '4'

            6) replace consecutive sequences of the nasals, (ie., the
            characters, M, N,) with the character '5'

            7) replace consecutive sequences of the shortliquids,
            (ie., the character, R,) with the character '6'

            8) and, omit all other characters, (ie., the characters,
            A, E, H, I, O, U, W, Y,)

            9) if the soundex translation of the word is larger than 4
            characters, truncate to 4 characters.

    For example, the soundex translation of the word "conover" is
    C516. Extended word length searches can be enhanced by eliminating
    9), above.

    Running the above algorithm on a standard text version of the
    Webster's English dictionary, (mine has 234,932 words,) results in
    61,408 different words being recognized by the algorithm.

$Revision: 1.0 $
$Date: 1998/03/09 00:29:06 $
$Id: pdict.c,v 1.0 1998/03/09 00:29:06 john Exp $
$Log: pdict.c,v $
Revision 1.0  1998/03/09 00:29:06  john
Initial revision


*/

#include "pdict.h"

#ifndef LINT /* include rcsid only if not running lint */

static char rcsid[] = "$Id: pdict.c,v 1.0 1998/03/09 00:29:06 john Exp $"; /* module version */
static char rcsid_h[] = PDICT_H_ID; /* module include version */

#endif

#ifdef __STDC__

const char *error_message[] = /* error message index array */

#else

char *error_message[] = /* error message index array */

#endif

{
    "",
    "Phonetic word lookup in the system dictionary\nUsage: pdict word [-d path] [-v]\n    -d path, dictionary file\n    -v, print the version and copyright banner of the program\n",
    "Error opening dictionary file\n",
    "Error closing dictionary file\n"
};

#define NOERROR 0 /* error values, one for each index in the error message array */
#define EARGS 1
#define EOPEN 2
#define ECLOSE 3

static char version[] = "$Revision: 1.0 $"; /* program version */

static char copyright[] = "Copyright (c) 1995, 1996, 1997, 1998 John Conover, All Rights Reserved"; /* the copyright banner */

#ifdef __STDC__

int main (int argc, char *argv[])

#else

int main (argc, argv)
    int argc;
    char *argv[];

#endif

{
    unsigned char criteria[BUFSIZ], /* copy of criteria argument from the command line */
                  default_dictionary[] = DICTIONARY_PATH, /* default dictionary path */
                  *dictionary = default_dictionary, /* dictionary path */
                  word[BUFSIZ], /* word from the dictionary */
                  word_copy[BUFSIZ]; /* copy of word from the dictionary */

    int c, /* command line switch */
        retval = EARGS; /* return value, assume argument error */

    FILE *dict; /* reference to dictionary file */

    (void) strcpy ((char *) dictionary, DICTIONARY_PATH); /* save the dictionary path */

    while ((c = getopt (argc, argv, "d:v")) != EOF) /* for each command line switch */
    {

        switch (c) /* which switch? */
        {

            case 'd': /* request for dictionary path? */

                dictionary = (unsigned char *) optarg; /* yes, save the dictionary path */
                break;

            case 'v': /* request for version */

                (void) printf ("%s\n", version); /* print the version number */
                (void) printf ("%s\n", copyright); /* print the copyright banner */
                optind = argc; /* force argument error */
                break;

            default: /* illegal switch? */

                optind = argc; /* force argument error */
                break;
        }

    }

    if (argc - optind > 0) /* enough arguments? */
    {
        (void) strcpy ((char *) criteria, argv[optind]); /* save the criteria arugment form the command line */
        make_uppercase (); /* setup the uppercase array */
        transliterate (criteria); /* translate criteria */
        retval = EOPEN; /* assume error opening file */

        if ((dict = fopen ((char *) dictionary, "r")) != (FILE *) 0)
        {
            retval = NOERROR; /* assume no error */

            while (fgets ((char *) word, BUFSIZ, dict) != (char *) 0) /* for each word in the dictionary */
            {
                word[strlen ((char *) word) - 1] = '\0'; /* remove the trailing EOL character for the word */
                (void) strcpy ((char *) word_copy, (char *) word); /* make a copy of the word */
                transliterate (word_copy); /* translate the copy of the word */

                if (strcmp ((char *) word_copy, (char *) criteria) == 0) /* if the translated word matches the criteria */
                {
                    (void) printf ("%s\n", word); /* print the word */
                }

            }

            retval = ECLOSE; /* assume error closing file */

            if (fclose (dict) == 0)
            {
                retval = NOERROR; /* assume no error */
            }

        }

    }

    (void) fprintf (stderr, "%s", error_message[retval]); /* print any errors */
    exit (retval); /* exit with the error value */

#ifdef LINT /* include only if running lint */

    return (retval); /* for LINT formality */

#endif

}
